comment ~
	Data Structure Collection
	 by x86 Assembly Language
	
	Linked-List Section
	
	Programmed by Woonsan Yeo, Paradoxoft Corporation.
~

.686
.mmx
.xmm
.model flat, stdcall

GetProcessHeap		proto
HeapAlloc			proto	hHeap:DWORD, dwFlags:DWORD, dwBytes:DWORD
HeapFree			proto	hHeap:DWORD, dwFlags:DWORD, lpMem:DWORD

LinkedListRegistrationStructure		struct
	FirstEntry			DWORD		?
	LastEntry			DWORD		?
	TotalEntryCount		DWORD		?
LinkedListRegistrationStructure		ends

LinkedListEntryStructure			struct
	ID					DWORD		?
	DataPoint			DWORD		?
	PreviousEntry		DWORD		?
	NextEntry			DWORD		?
LinkedListEntryStructure			ends

.code
	; Management Operation
	comment !
		Linked-List Registration - Set structure information
		 Name		/	RegisterLinkedList
		 Parameters /	StructureAddress - Address of Linked-List Registration Structure !
	RegisterLinkedList		proc	StructureAddress:DWORD
		pushad
		mov eax, dword ptr [StructureAddress]
		mov dword ptr [(LinkedListRegistrationStructure ptr [eax]).FirstEntry], 0			; Set [FirstEntry]
		mov dword ptr [(LinkedListRegistrationStructure ptr [eax]).LastEntry], 0			; Set [LastEntry]
		mov dword ptr [(LinkedListRegistrationStructure ptr [eax]).TotalEntryCount], 0		; Set [TotalEntryCount]
		popad
		ret
	RegisterLinkedList		endp
	
	comment!
		Linked-List Release - Release linked-list entries and clear structure information
		Name		/	ReleaseLinkedList
		Parameters	/	StructureAddress - Address of Linked-List Registration Structure !
	ReleaseLinkedList		proc	StructureAddress:DWORD
		pushad
		invoke GetProcessHeap
		mov edx, eax																		; Store Process Heap into EDX
		mov eax, dword ptr [StructureAddress]
		mov ebx, dword ptr [(LinkedListRegistrationStructure ptr [eax]).FirstEntry]			; Get [FirstEntry]
		or ebx, ebx																			; If the first entry does not exist
		jz ReleaseStructure																	; Just release Linked-List Registration Structure
		ReleaseEntry:
			mov ecx, dword ptr [(LinkedListEntryStructure ptr [ebx]).NextEntry]				; Get [PreviousEntry]
			pushad
			invoke HeapFree, edx, 0, ebx													; Disallocate current entry
			popad
			or ecx, ecx
			jz ReleaseStructure																; If the last entry
			mov ebx, ecx
		jmp ReleaseEntry
		ReleaseStructure:
		mov eax, dword ptr [StructureAddress]
		mov dword ptr [(LinkedListRegistrationStructure ptr [eax]).FirstEntry], 0			; Set [FirstEntry]
		mov dword ptr [(LinkedListRegistrationStructure ptr [eax]).LastEntry], 0			; Set [LastEntry]
		mov dword ptr [(LinkedListRegistrationStructure ptr [eax]).TotalEntryCount], 0		; Set [TotalEntryCount]
		popad
		ret
	ReleaseLinkedList		endp
	
	; Data Operation
	AddEntry				proc	StructureAddress:DWORD, ID:DWORD, DataPoint:DWORD
		pushad
		invoke GetProcessHeap
		invoke HeapAlloc, eax, 0, 16														; Allocate an entry
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(LinkedListRegistrationStructure ptr [ebx]).FirstEntry]			; Get [FirstEntry]
		or ecx, ecx																			; If first entry exists
		jnz	PassFirstEntry																	; Pass the first entry
		mov dword ptr [(LinkedListRegistrationStructure ptr [ebx]).FirstEntry], eax			; Set [FirstEntry]
		PassFirstEntry:
		mov ecx, dword ptr [(LinkedListRegistrationStructure ptr [ebx]).LastEntry]			; Get [LastEntry]
		or ecx, ecx																			; If last entry exists
		jz PassLastEntry																	; Pass the last entry
		mov dword ptr [(LinkedListEntryStructure ptr [ecx]).NextEntry], eax					; Set [NextEntry]
		PassLastEntry:
		mov dword ptr [(LinkedListRegistrationStructure ptr [ebx]).LastEntry], eax			; Set [LastEntry] as new entry
		inc dword ptr [(LinkedListRegistrationStructure ptr [ebx]).TotalEntryCount]			; Increase [TotalEntryCount]
		mov dword ptr [(LinkedListEntryStructure ptr [eax]).PreviousEntry], ecx				; Set [PreviousEntry]
		mov dword ptr [(LinkedListEntryStructure ptr [eax]).NextEntry], 0					; Set [NextEntry]
		mov edx, dword ptr [ID]
		mov (LinkedListEntryStructure ptr [eax]).ID, edx									; Set [ID]
		mov edx, dword ptr [DataPoint]
		mov (LinkedListEntryStructure ptr [eax]).DataPoint, edx								; Set [DataPoint]
		popad
		ret
	AddEntry				endp
	
	comment %
	InsertEntryNext			proc	StructureAddress:DWORD, ID:DWORD, DataPoint:DWORD, MainEntry:DWORD
		ret
	InsertEntryNext			endp
	
	InsertEntryPrevious		proc	StructureAddress:DWORD, ID:DWORD, DataPoint:DWORD, MainEntry:DWORD
		ret
	InsertEntryPrevious		endp
	%
	
	DeleteEntry				proc	StructureAddress:DWORD, RunEntry:DWORD
		pushad
		mov eax, dword ptr [RunEntry]
		mov edx, dword ptr [StructureAddress]
		mov ebx, dword ptr [(LinkedListEntryStructure ptr [eax]).NextEntry]					; Get [NextEntry] Of RunEntry
		mov ecx, dword ptr [(LinkedListEntryStructure ptr [eax]).PreviousEntry]				; Get [PreviousEntry] Of RunEntry
		or ecx, ecx																			; If the first entry
		jnz NotFirstEntry																	; Pass
		mov dword ptr [(LinkedListRegistrationStructure ptr [edx]).FirstEntry], ebx			; Set [FirstEntry] as [NextEntry] Of RunEntry
		NotFirstEntry:
		or ebx, ebx																			; If the last entry
		jz PassPreviousEntry																; Pass
		mov dword ptr [(LinkedListEntryStructure ptr [ebx]).PreviousEntry], ecx				; Set [PreviousEntry] Of [NextEntry] Of RunEntry
		PassPreviousEntry:
		or ecx, ecx
		jz PassNextLastEntry
		mov dword ptr [(LinkedListEntryStructure ptr [ecx]).NextEntry], ebx					; Set [NextEntry] Of [PreviousEntry] Of RunEntry
		or ebx, ebx																			; If the last entry
		jnz PassNextLastEntry																; Pass
		mov dword ptr [(LinkedListRegistrationStructure ptr [edx]).LastEntry], ecx			; Set [LastEntry] as [NextEntry] Of RunEntry
		PassNextLastEntry:
		dec dword ptr [(LinkedListRegistrationStructure ptr [edx]).TotalEntryCount]			; Decrease [FirstEntry]
		invoke GetProcessHeap
		invoke HeapFree, eax, 0, dword ptr [RunEntry]										; Disallocate RunEntry
		popad
		ret
	DeleteEntry				endp
	
	SearchEntry				proc	StructureAddress:DWORD, ID:DWORD
		push ebx
		push ecx
		push edx
		push esi
		push edi
		mov esi, dword ptr [ID]
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(LinkedListRegistrationStructure ptr [ebx]).FirstEntry]
		SearchLoop:
			mov edx, dword ptr [(LinkedListEntryStructure ptr [ecx]).ID]
			xor edx, esi																	; If find corresponding entry
			jz EndSearch																	; End loop
			mov edi, dword ptr [(LinkedListEntryStructure ptr [ecx]).NextEntry]				; Get [NextEntry]
			or edi, edi
			jz NoData
			mov ecx, edi
		jmp SearchLoop
		NoData:
			mov eax, -1																		; Return not found
			jmp Final
		EndSearch:
			mov eax, ecx																	; Return address of corresponding structure
		Final:
		pop edi
		pop esi
		pop edx
		pop ecx
		pop ebx
		ret
	SearchEntry				endp

end